@@ -1,6 +1,6 @@
 /************************************************************
  *
- * COPYRIGHT (C) HITACHI,LTD. 2002-2003
+ * COPYRIGHT (C) HITACHI,LTD. 2002-2004
  * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
  *            HITACHI CENTRAL RESEARCH LABORATORY.
  *
@@ -30,9 +30,9 @@
 #define LKST_ETYPE_DEF_DESCRIPTIONS
 #include <linux/lkst_events.h>
 
-#define COPYRIGHT "COPYRIGHT (C) HITACHI,LTD. 2002-2003"
+#define COPYRIGHT "COPYRIGHT (C) HITACHI,LTD. 2002-2004"
 #define NAME	"lkstlogd"
-#define VERSION "1.5"
+#define VERSION "1.6"
 
 #define LOG_HEADER_SIZE \
 	(sizeof(log_header_t) +					\
@@ -179,6 +179,10 @@ void (*_errlog)(const char *format, ...) = stderr_error;
 #define DEBUG_LOG(fmt, args...)
 #endif
 
+#define BAD_COND_THRESHOLD_SEC 2
+time_t child_death_time = 0;
+int child_killing = 0; /*death check flag*/
+
 int __inline__ buffer_read_size(int buf_size)
 {
 	return buf_size * sizeof(struct lkst_log_record) / LKST_SIZEOF_LKST_EVENT_RECORD;
@@ -260,6 +264,8 @@ int owner_daemon()
 			errlog( "Cannot register signal handlers\n");
 			exit(1);
 		}
+		/* there is no reason for killing children */
+		child_killing = 0;
 
 		/* fork children */
 		lkst_for_each_online_cpu(i, prev_status){
@@ -286,6 +292,22 @@ int owner_daemon()
 			exit(1);
 		}
 
+		if(!child_killing) { 
+			time_t tm;
+			/* Owner doesn't kill children explicitly. So, 
+			 * something bad might be occurred.*/
+			tm = time(NULL);
+			errlog("Child processes died without being killed by the owner.");
+			if (child_death_time != 0 &&
+			    tm - child_death_time < BAD_COND_THRESHOLD_SEC) {
+				/* Children die so frequently, we are under 
+				   bad condition. */
+				restart = 0;
+				errlog("Child process might be under bad condition.");
+			} else {
+				child_death_time = tm;
+			}
+		}
 	} while (restart);
 
 	/* close device */
@@ -662,14 +684,16 @@ void cleanup(void)
 /* signal handlers */
 void owner_hup_handler(int s)
 {
-	kill_children(SIGTERM);
 	restart = 1;
+	child_killing = 1;
+	kill_children(SIGTERM);
 }
 
 void owner_die_handler(int s)
 {
-	kill_children(SIGTERM);
 	restart = 0;
+	child_killing = 1;
+	kill_children(SIGTERM);
 }
 
 void owner_child_handler(int s)
@@ -693,17 +717,17 @@ void child_die_handler(int s)
 	term_flg = 1;
 }
 
+void child_snap_handler(int s)
+{
+	read_on = !read_on;
+}
+
 void child_switch_handler(int s)
 {
 	file_snum++;
 	switchlog = 1;
 }
 
-void child_snap_handler(int s)
-{
-	read_on = !read_on;
-}
-
 void buffer_setrmod(int devfd)
 {
 	int ret;
